<?php

namespace Ems\Repository;

use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;
use Ems\Util\Utils;

class GeralRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function getPaisByNameOrSg(
        $nmPais = null,
        $sgPais = null
    ) {
        $sql = <<<SQL
            SELECT *
            FROM PAIS
            WHERE
                UPPER(NMPAIS) = UPPER(:P_NMPAIS) OR
                UPPER(SGPAIS) = UPPER(:P_SGPAIS)
SQL;

        if ($nmPais) {
            $nmPais = Utils::removeCaracteresEspeciais($nmPais);
        }

        $params = [
            'P_NMPAIS' => $nmPais ?? null,
            'P_SGPAIS' => $sgPais ?? null,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getEstadoByNameOrSg(
        string $cdPais,
        $nmEstado = null,
        $sgEstado = null
    ) {
        $sql = <<<SQL
            SELECT *
            FROM ESTADO
            WHERE
                CDPAIS = :P_CDPAIS AND
                (
                    UPPER(NMESTADO) = UPPER(:P_NMESTADO) OR
                    UPPER(SGESTADO) = UPPER(:P_SGESTADO)
                )
SQL;

        if ($nmEstado) {
            $nmEstado = Utils::removeCaracteresEspeciais($nmEstado);
        }

        $params = [
            'P_CDPAIS' => $cdPais,
            'P_NMESTADO' => $nmEstado ?? null,
            'P_SGESTADO' => $sgEstado ?? null,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getCidadeByName(
        string $cdPais,
        string $sgEstado,
        string $nmMunicipio
    ) {
        $sql = <<<SQL
            SELECT *
            FROM MUNICIPIO
            WHERE
                CDPAIS = :P_CDPAIS AND
                SGESTADO = :P_SGESTADO AND
                UPPER(NMMUNICIPIO) = UPPER(:P_NMMUNICIPIO)
SQL;

        $nmMunicipio = Utils::removeCaracteresEspeciais($nmMunicipio);

        $params = [
            'P_CDPAIS' => $cdPais,
            'P_SGESTADO' => $sgEstado,
            'P_NMMUNICIPIO' => $nmMunicipio,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getBairroByName(
        string $cdPais,
        string $sgEstado,
        string $cdMunicipio,
        string $nmBairro
    ) {
        $sql = <<<SQL
            SELECT *
            FROM BAIRRO
            WHERE
                CDPAIS = :P_CDPAIS AND
                SGESTADO = :P_SGESTADO AND
                CDMUNICIPIO = :P_CDMUNICIPIO AND
                UPPER(NMBAIRRO) = UPPER(:P_NMBAIRRO)
SQL;

        $nmBairro = Utils::removeCaracteresEspeciais($nmBairro);

        $params = [
            'P_CDPAIS' => $cdPais,
            'P_SGESTADO' => $sgEstado,
            'P_CDMUNICIPIO' => $cdMunicipio,
            'P_NMBAIRRO' => $nmBairro,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getLogradouroByName(
        string $logradouro
    ): array {
        $sql = <<<SQL
            SELECT *
            FROM LOGRADOURO
            WHERE
                UPPER(DSLOGRADOURO) = :P_LOGRADOURO OR
                UPPER(DSABREVLOGRA) = :P_LOGRADOURO
SQL;

        $params = [
            'P_LOGRADOURO' => $logradouro,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }

    public function getNacionalidadeByCdPais(
        string $cdPais
    ): array {
        $sql = <<<SQL
            SELECT *
            FROM GPE_NACIONALIDADE
            WHERE
                CDPAIS = :P_CDPAIS
SQL;

        $params = [
            'P_CDPAIS' => $cdPais,
        ];

        $std = $this->connection->prepare($sql);
        $std->execute($params);

        $result = $std->fetch();

        return $result;
    }
}