<?php

namespace Ems\Repository;

use Ems\Util\Utils;
use Doctrine\ORM\EntityManager;
use Doctrine\DBAL\Connection;

class ProcessoRepository {
    /** @var Connection */
    protected $connection;

    public function __construct(
        EntityManager $entityManager
    ) {
        $this->connection = $entityManager->getConnection();
    }

    public function get(
        int $nrOrg,
        int $nrProcesso
    ) {
        $sql = <<<SQL
            SELECT
                *
            FROM
                DES_PROCESSO PRO
            WHERE
                PRO.NRPROCESSO = :P_NRPROCESSO AND
                PRO.NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPROCESSO' => $nrProcesso,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetch();
        return $result;
    }

    public function getStatusProcesso(
        int $nrOrg,
        int $nrOrgPadrao
    ): array {
        $sql = <<<SQL
            SELECT
              CDSTATUS,
              DSSTATUS
              FROM DES_STATUS
             WHERE (NRORG = :P_NRORG_PADRAO OR NRORG = :P_NRORG)
             ORDER BY DSSTATUS
SQL;

        $params = [
            'P_NRORG'           => $nrOrg,
            'P_NRORG_PADRAO'    => $nrOrgPadrao,
        ];

        $std = Utils::getEntityManager()->getConnection()->prepare($sql);
        $std->execute($params);

        $result = $std->fetchAll();

        return $result;
    }

    public function getProcessosGestor(
        int $nrOrg,
        int $nrParcNegocio
    ): array {
        $sql = <<<SQL
            SELECT
                PRO.*,
                TO_CHAR(PRO.DTLIMSOLICITA, 'DD/MM/YYYY') DTLIMSOLICITA,
                TO_CHAR(PRO.DTSOLICITA, 'DD/MM/YYYY') DTSOLICITA,
                RESP.NMPRINCIPALPARC NMRESPONSAVEL,
                SOLI.NMPRINCIPALPARC NMSOLICITA,
                DES_TEMPLATE.NMTEMPLATE,
                DES_STATUS.DSSTATUS,
                OCUPA.NMOCUPACAOH,
                ESTRUTURAH.NMESTRUTURAH,
                ESCALA.NMESCALATRABH,
                DES_FASE.NRFASE,
                DES_FASE.NMFASE
            FROM
                DES_PROCESSO PRO
                LEFT JOIN DES_TIPOPROCESSO TIPO ON
                    TIPO.NRTIPOPROCESSO = PRO.NRTIPOPROCESSO AND
                    TIPO.NRORG = PRO.NRORG
                LEFT JOIN PARCNEGOCIO RESP ON
                    RESP.NRPARCNEGOCIO = PRO.NRPARCNEGOCIORESP AND
                    RESP.NRORG = PRO.NRORG
                LEFT JOIN PARCNEGOCIO SOLI ON
                    SOLI.NRPARCNEGOCIO = PRO.NRPARCNEGOCIOSOLICITA AND
                    SOLI.NRORG = PRO.NRORG
                LEFT JOIN DES_TEMPLATE ON
                    DES_TEMPLATE.NRTEMPLATE = PRO.NRTEMPLATE AND
                    DES_TEMPLATE.NRORG = PRO.NRORG
                LEFT JOIN DES_STATUS ON
                    DES_STATUS.CDSTATUS = PRO.IDSTATUS
                LEFT JOIN GPE_OCUPACAOH OCUPA ON
                    OCUPA.DTMESCOMPETENC = (
                        SELECT MAX(OC1.DTMESCOMPETENC)
                        FROM GPE_OCUPACAOH OC1
                        WHERE
                            OC1.DTMESCOMPETENC <= SYSDATE AND
                            OC1.NROCUPACAOM = PRO.NROCUPACAODESEJADA AND
                            OC1.NRORG = PRO.NRORG
                    ) AND
                    OCUPA.NROCUPACAOM = PRO.NROCUPACAODESEJADA AND
                    OCUPA.NRORG = PRO.NRORG
                LEFT JOIN ESTRUTURAH ON
                    ESTRUTURAH.DTMESCOMPETENC = (
                        SELECT MAX(E1.DTMESCOMPETENC)
                        FROM ESTRUTURAH E1
                        WHERE
                            E1.DTMESCOMPETENC <= SYSDATE AND
                            E1.NRESTRUTURAM = PRO.NRESTRUTGEREN AND
                            E1.NRORG = PRO.NRORG
                    ) AND
                    ESTRUTURAH.NRESTRUTURAM = PRO.NRESTRUTGEREN AND
                    ESTRUTURAH.NRORG = PRO.NRORG
                LEFT JOIN GPE_ESCALATRABH ESCALA ON
                    ESCALA.DTMESCOMPETENC = (
                        SELECT MAX(ESC1.DTMESCOMPETENC)
                        FROM GPE_ESCALATRABH ESC1
                        WHERE
                            ESC1.DTMESCOMPETENC <= SYSDATE AND
                            ESC1.NRESCALATRABM = PRO.NRESCALATRABM AND
                            ESC1.NRORG = PRO.NRORG
                    ) AND
                    ESCALA.NRESCALATRABM = PRO.NRESCALATRABM AND
                    ESCALA.NRORG = PRO.NRORG
                LEFT JOIN DES_FASE ON
                    DES_FASE.NRFASE = (
                        SELECT MAX(F1.NRFASE)
                        FROM
                            DES_FASE F1
                            JOIN DES_PARTICIPAFASE PF1 ON
                                PF1.NRFASE = F1.NRFASE AND
                                PF1.NRPROCESSO = PRO.NRPROCESSO AND
                                PF1.NRORG = F1.NRORG
                    ) AND
                    DES_FASE.NRORG = PRO.NRORG
            WHERE
                :P_NRPARC IN (PRO.NRPARCNEGOCIOSOLICITA, PRO.NRPARCNEGOCIORESP) AND
                PRO.NRORG = :P_NRORG
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRPARC' => $nrParcNegocio,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }

    public function getFasesTemplate(
        int $nrOrg,
        int $nrTemplate
    ): array {
        $sql = <<<SQL
            SELECT
                IDINATIVARPROCESSO,
                NRORDEM,
                NRFASETEMPLATE,
                FT.NRORG,
                FT.NRTEMPLATE,
                FT.NRFASE,
                F.NMFASE,
                FT.NRDURACAO,
                FT.NRPESO,
                FT.IDOBRIGATORIO,
                NRFASERELACIONADA,
                (
                    SELECT
                        NMFASE
                    FROM
                        DES_FASE
                    WHERE
                            NRORG = FT.NRORG
                        AND NRFASE = NRFASERELACIONADA
                ) AS NMFASERELACIONADA,
                F.DTINCLUSAO,
                F.DTULTATU,
                F.NRORGINCLUSAO,
                F.CDOPERINCLUSAO,
                F.NRORGULTATU,
                F.CDOPERULTATU,
                PF.NRCRITERIORESUL,
                CR.NMCRITERIORESUL,
                PF.VRFASE,
                (
                    SELECT
                        MAX(NRORDEM)
                    FROM
                        DES_FASETEMPLATE
                    WHERE
                            NRTEMPLATE = FT.NRTEMPLATE
                        AND NRORG = FT.NRORG
                ) AS MAXNRORDEM
            FROM
                    DES_FASETEMPLATE FT
                JOIN DES_FASE       F ON FT.NRFASE = F.NRFASE
                                AND FT.NRORG = F.NRORG
                JOIN DES_TEMPLATE   T ON FT.NRTEMPLATE = T.NRTEMPLATE
                                    AND FT.NRORG = T.NRORG
                LEFT JOIN DES_PARAMFASE  PF ON FT.NRFASE = PF.NRFASE
                                            AND FT.NRORG = PF.NRORG
                LEFT JOIN CRITERIORESULT CR ON PF.NRCRITERIORESUL = CR.NRCRITERIORESUL
                                            AND FT.NRORG = CR.NRORG
            WHERE
                    FT.NRTEMPLATE = :P_NRTEMPLATE
                AND FT.NRORG = :P_NRORG
            ORDER BY
                NRORDEM
SQL;

        $params = [
            'P_NRORG' => $nrOrg,
            'P_NRTEMPLATE' => $nrTemplate,
        ];

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }

    public function getCandidatosFase(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                PF.NRORG,
                PESSOA.NRORG NRORG_CAND,
                PF.NRPARCNEGOCIOCAND,
                PF.NRPARCTICIPAFASE,
                PF.NRFASE,
                PF.NRPROCESSO,
                PF.DTINICIO,
                PF.DTFIM,
                PF.NRPONTUACAO,
                PF.NRAVALIACAO,
                PF.NRDURACAO,
                PF.NRPESO,
                PF.IDOBRIGATORIO,
                PF.DSLOCALFASE,
                PF.IDSTATUS,
                PF.DTINCLUSAO,
                PF.DTULTATU,
                PF.NRORGINCLUSAO,
                PF.CDOPERINCLUSAO,
                PF.NRORGULTATU,
                PF.CDOPERULTATU,
                S.DSSTATUS,
                P.NMPRINCIPALPARC,
                PF.CDTIPORESU,
                TP.NMTIPORESU,
                DECODE(PF.NRPERCENTUAL, NULL, NULL, PF.NRPERCENTUAL || '%')       NRPERCENTUAL,
                PESSOAH.NRCPFPESSOA,
                DECODE(PC.VRMEDIAPROCESSO, NULL, NULL, PC.VRMEDIAPROCESSO || '%') VRMEDIAPROCESSO
            FROM
                    DES_PARTICIPAFASE PF
                JOIN PARCNEGOCIO           P ON PF.NRPARCNEGOCIOCAND = P.NRPARCNEGOCIO
                                    AND P.NRORG = :P_NRORGPADRAO
                LEFT JOIN DES_STATUS            S ON PF.IDSTATUS = S.CDSTATUS
                LEFT JOIN TIPORESU              TP ON PF.CDTIPORESU = TP.CDTIPORESU
                                        AND PF.NRORG = TP.NRORG
                LEFT JOIN GPE_PESSOA            PESSOA ON PESSOA.NRPARCNEGOCIO = PF.NRPARCNEGOCIOCAND
                                            AND PESSOA.NRORG = P.NRORG
                LEFT JOIN GPE_PESSOAH           PESSOAH ON PESSOAH.NRPESSOA = PESSOA.NRPESSOA
                                                AND PESSOAH.NRORG = PESSOA.NRORG
                LEFT JOIN DES_PARTICIPAPROCESSO PC ON PC.NRPARCNEGOCIOCAND = PF.NRPARCNEGOCIOCAND
                                                    AND PC.NRPROCESSO = PF.NRPROCESSO
                                                    AND PC.NRORG = PF.NRORG
            WHERE
                    PF.NRORG = :P_NRORG
                AND PF.NRPROCESSO = :P_NRPROCESSO
                AND PF.NRFASE = :P_NRFASE
            ORDER BY PESSOAH.NMPESSOA
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();

        return $result;
    }

    public function getCandidatosGeral(
        array $params
    ): array {
        $sql = <<<SQL
            SELECT
                P.NRORG,
                P.NRPESSOA,
                PH.NRPESSOAH,
                P.NRPARCNEGOCIO,
                PH.NMPESSOA,
                FLOOR(MONTHS_BETWEEN(TRUNC(SYSDATE), PH.DTNASCPESSOA) /12) IDADE,
                (MUNICIPIO.NMMUNICIPIO || ' - ' || EP.SGESTADO) LOCALIDADE,
                GPE_GRAUINSTR.NMGRAUINSTR
            FROM
                GPE_PESSOA P
                JOIN GPE_PESSOAH PH ON
                PH.DTMESCOMPETENC = (
                    SELECT MAX(PH1.DTMESCOMPETENC)
                    FROM GPE_PESSOAH PH1
                    WHERE
                        PH1.DTMESCOMPETENC <= TRUNC(SYSDATE) AND
                        PH1.NRPESSOA = P.NRPESSOA AND
                        PH1.NRORG = P.NRORG
                ) AND
                PH.NRPESSOA = P.NRPESSOA AND
                PH.NRORG = P.NRORG
                LEFT JOIN ENDERECOPARC EP ON
                    EP.IDATIVO = 'S' AND
                    EP.CDTIPOENDERECO = 'PRINCIPAL' AND
                    EP.NRPARCNEGOCIO = P.NRPARCNEGOCIO AND
                    EP.NRORG = P.NRORG
                LEFT JOIN MUNICIPIO ON
                    MUNICIPIO.CDMUNICIPIO = EP.CDMUNICIPIO AND
                    MUNICIPIO.SGESTADO = EP.SGESTADO
                LEFT JOIN GPE_GRAUINSTR ON
                    GPE_GRAUINSTR.NRGRAUINSTR = PH.NRGRAUINSTR
            WHERE
                (
                    UPPER(PH.NMPESSOA) LIKE UPPER(:NMPESSOA) OR
                    :NMPESSOA IS NULL
                ) AND
                (
                    EXISTS (
                        SELECT 1
                        FROM COMUNICAPARC CP
                        WHERE
                            UPPER(CP.CDCOMUNICAPARC) = UPPER(:EMAIL) AND
                            CP.NRPARCNEGOCIO = P.NRPARCNEGOCIO AND
                            CP.NRORG = P.NRORG
                    ) OR
                    :EMAIL IS NULL
                ) AND
                (
                    PH.DTNASCPESSOA = :DTNASCPESSOA OR
                    :DTNASCPESSOA IS NULL
                ) AND
                (
                    :IDADE = FLOOR(MONTHS_BETWEEN(TRUNC(SYSDATE), PH.DTNASCPESSOA) /12) OR
                    :IDADE IS NULL
                ) AND
                (
                    :IDADEMINIMA <= FLOOR(MONTHS_BETWEEN(TRUNC(SYSDATE), PH.DTNASCPESSOA) /12) OR
                    :IDADEMINIMA IS NULL
                ) AND
                (
                    :IDADEMAXIMA >= FLOOR(MONTHS_BETWEEN(TRUNC(SYSDATE), PH.DTNASCPESSOA) /12) OR
                    :IDADEMAXIMA IS NULL
                ) AND
                (
                    :IDSEXOPESSOA = PH.IDSEXOPESSOA OR
                    :IDSEXOPESSOA IS NULL
                ) AND
                (
                    EXISTS (
                        SELECT 1
                        FROM DES_PARTICIPAPROCESSO PART
                        WHERE
                            (
                                PART.VRPRETENSAOSALARIO >= :SALARIOMINIMO OR
                                :SALARIOMINIMO IS NULL
                            ) AND
                            (
                                PART.VRPRETENSAOSALARIO <= :SALARIOMAXIMO OR
                                :SALARIOMAXIMO IS NULL
                            ) AND
                            PART.NRPARCNEGOCIOCAND = P.NRPARCNEGOCIO AND
                            PART.NRORG = :NRORG
                    ) OR
                    (
                        :SALARIOMINIMO IS NULL AND
                        :SALARIOMAXIMO IS NULL
                    )
                ) AND
                (
                    :CDESTACIVIL = PH.CDESTACIVIL OR
                    :CDESTACIVIL IS NULL
                ) AND
                (
                    :NRNACIONALID = PH.NRNACIONALID OR
                    :NRNACIONALID IS NULL
                ) AND
                (
                    :IDPOSSUIFILHO = PH.IDPOSSUIFILHO OR
                    :IDPOSSUIFILHO IS NULL
                ) AND
                (
                    EXISTS (
                        SELECT 1
                        FROM DES_FORMACAO
                        WHERE
                            (
                                :NRCURSO = DES_FORMACAO.NRCURSO OR
                                :NRCURSO IS NULL
                            ) AND
                            (
                                UPPER(DES_FORMACAO.DSCURSO) LIKE UPPER(:DSCURSO)  OR
                                :DSCURSO IS NULL
                            ) AND
                            (
                                UPPER(DES_FORMACAO.DSINSTITUICAO) LIKE UPPER(:DSINSTITUICAO) OR
                                :DSINSTITUICAO IS NULL
                            ) AND
                            (
                                :NRAREAFORMACAO = DES_FORMACAO.NRAREAFORMACAO OR
                                :NRAREAFORMACAO IS NULL
                            ) AND
                            (
                                UPPER(DES_FORMACAO.DES_AREAFORMACAO_NRORG) LIKE UPPER(:DSAREAFORMACAO) OR
                                :DSAREAFORMACAO IS NULL
                            ) AND
                            DES_FORMACAO.NRPARCNEGOCIO = P.NRPARCNEGOCIO AND
                            DES_FORMACAO.NRORG = P.NRORG
                    ) OR
                    (
                        :NRCURSO IS NULL AND
                        :DSCURSO IS NULL AND
                        :DSINSTITUICAO IS NULL AND
                        :NRAREAFORMACAO IS NULL AND
                        :DSAREAFORMACAO IS NULL
                    )
                ) AND
                (
                    EXISTS (
                        SELECT 1
                        FROM ENDERECOPARC ENDE
                        WHERE
                            ENDE.NRPARCNEGOCIO = P.NRPARCNEGOCIO AND
                            :CDPAIS = ENDE.CDPAIS AND
                            (
                                :SGESTADO = ENDE.SGESTADO OR
                                :SGESTADO IS NULL
                            ) AND
                            (
                                :CDMUNICIPIO = ENDE.CDMUNICIPIO OR
                                :CDMUNICIPIO IS NULL
                            )
                    ) OR
                    (
                        :CDPAIS IS NULL AND
                        :SGESTADO IS NULL AND
                        :CDMUNICIPIO IS NULL
                    )
                ) AND
                (
                    EXISTS (
                        SELECT 1
                        FROM GPE_EMPREGANTERIOR EA
                        WHERE
                            (
                                :NROCUPACAOM = EA.NROCUPACAOM OR
                                :NROCUPACAOM IS NULL
                            ) AND
                            (
                                UPPER(EA.DSOCUPACAO) LIKE UPPER(:DSOCUPACAO) OR
                                :DSOCUPACAO IS NULL
                            ) AND
                            EA.NRPARCNEGOCIO = P.NRPARCNEGOCIO AND
                            EA.NRORG = P.NRORG
                    ) OR
                    (
                        :NROCUPACAOM IS NULL AND
                        :DSOCUPACAO IS NULL
                    )
                ) AND
                (
                    (SELECT DECODE(SUM(STATUS), 5, 'N', 'S') STATUS
                            FROM (
                            SELECT
                            CASE
                                WHEN(COUNT(1) > 0)
                                THEN 1
                            END AS STATUS
                            FROM DES_APRESENTACAOCAND
                            WHERE NRPARCNEGOCIO = P.NRPARCNEGOCIO
                            AND NRORG           = P.NRORG
                            UNION ALL
                            SELECT 
                            CASE
                                WHEN(COUNT(1) > 0)
                                THEN 1
                            END AS STATUS
                            FROM ENDERECOPARC
                            WHERE NRPARCNEGOCIO = P.NRPARCNEGOCIO
                            AND NRORG           = P.NRORG
                            UNION ALL
                            SELECT
                            CASE
                                WHEN(COUNT(1) > 0)
                                THEN 1
                            END AS STATUS
                            FROM COMUNICAPARC
                            WHERE NRPARCNEGOCIO = P.NRPARCNEGOCIO
                            AND NRORG           = P.NRORG
                            AND CDFORMACOMU    != '05'
                            UNION ALL
                            SELECT 
                            CASE
                                WHEN (COUNT(1) > 0)
                                THEN 1
                            END AS STATUS
                            FROM DES_FORMACAO
                            WHERE NRPARCNEGOCIO = P.NRPARCNEGOCIO
                            AND NRORG           = P.NRORG
                            UNION ALL
                            SELECT 
                            CASE
                                WHEN (COUNT(1) > 0)
                                THEN 1
                            END AS STATUS
                            FROM GPE_EMPREGANTERIOR
                            WHERE NRPARCNEGOCIO = P.NRPARCNEGOCIO
                            AND NRORG           = P.NRORG
                            )) = :IDCADASTROCOMPLETO OR
                            :IDCADASTROCOMPLETO IS NULL
                ) AND
                (
                    NOT EXISTS (
                        SELECT 1
                        FROM DES_PARTICIPAPROCESSO PP1
                        WHERE
                            PP1.NRPROCESSO = :NRPROCESSO AND
                            PP1.NRPARCNEGOCIOCAND = P.NRPARCNEGOCIO AND
                            PP1.NRORG = :NRORG
                    ) OR
                    :NRPROCESSO IS NULL
                ) AND
                P.NRORG = :NRORG_PADRAO
            ORDER BY PH.NMPESSOA ASC
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);

        $result = $stmt->fetchAll();
        return $result;
    }

    public function setParticipaProcesso(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO DES_PARTICIPAPROCESSO (
                NRORG,                  NRPROCPARTICIPA,    NRPARCNEGOCIOCAND,
                NRPARCINDICACAO,        NRORIGEM,           NRPROCESSO,
                DTINCLUSAO,             DTULTATU,           NRORGINCLUSAO,
                CDOPERINCLUSAO,         NRORGULTATU,        CDOPERULTATU,
                CDSTATUS,               DSOBSERVACAOCAND,   VRPRETENSAOSALARIO,
                VRPRETENCAOSALARIO,     VRMEDIAPROCESSO,    NRVINCULOM,
                IDPARECERFINAL
            ) VALUES (
                :NRORG,                 :NRPROCPARTICIPA,   :NRPARCNEGOCIOCAND,
                :NRPARCINDICACAO,       :NRORIGEM,          :NRPROCESSO,
                SYSDATE,                SYSDATE,            :NRORG,
                :CDOPERADOR,            :NRORG,             :CDOPERADOR,
                :CDSTATUS,              :DSOBSERVACAOCAND,  :VRPRETENSAOSALARIO,
                :VRPRETENCAOSALARIO,    :VRMEDIAPROCESSO,   :NRVINCULOM,
                :IDPARECERFINAL
            )
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    public function setParticipaFase(
        array $params
    ): void {
        $sql = <<<SQL
            INSERT INTO DES_PARTICIPAFASE (
                NRPARCTICIPAFASE,       NRORG,              NRFASE,
                NRPARCNEGOCIOCAND,      NRPROCESSO,         DTINICIO,
                DTFIM,                  NRPONTUACAO,        NRPERCENTUAL,
                NRAVALIACAO,            NRDURACAO,          NRPESO,
                IDOBRIGATORIO,          DSLOCALFASE,        IDSTATUS,
                CDTIPORESU,             DTINCLUSAO,         DTULTATU,
                NRORGINCLUSAO,          CDOPERINCLUSAO,     NRORGULTATU,
                CDOPERULTATU
            ) VALUES (
                :NRPARCTICIPAFASE,      :NRORG,             :NRFASE,
                :NRPARCNEGOCIOCAND,     :NRPROCESSO,        :DTINICIO,
                :DTFIM,                 :NRPONTUACAO,       :NRPERCENTUAL,
                :NRAVALIACAO,           :NRDURACAO,         :NRPESO,
                :IDOBRIGATORIO,         :DSLOCALFASE,       :IDSTATUS,
                :CDTIPORESU,            SYSDATE,            SYSDATE,
                :NRORG,                 :CDOPERADOR,        :NRORG,
                :CDOPERADOR
            )
SQL;

        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
}