<?php

namespace Mig\Controllers;

use Mig\Services\MigracaoPlanilhaService;
use Util\Environment;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };
use Zeedhi\Framework\DTO\Response\Notification;


class MigracaoPlanilhaController {
    /** @var MigracaoPlanilhaService */
    protected $migracaoPlanilhaService;

    public function __construct(
        Environment $environment,
        MigracaoPlanilhaService $migracaoPlanilhaService
    ) {
        $this->environment                  = $environment;
        $this->migracaoPlanilhaService      = $migracaoPlanilhaService;
    }
    
    public function importarPlanilha(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $this->migracaoPlanilhaService->importarPlanilha($row);
            
            $hasErrors = !empty($this->migracaoPlanilhaService->errosImportacaoPlanilha());
            
            if ($hasErrors) {
                $response->addNotification(new Notification("A importação possui pendências!", Notification::TYPE_ERROR));
                
                return $response->addDataSet(new DataSet('errosImportacao', []));
            }
            
            $response->addDataSet(new DataSet('importarPlanilha', []));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }
    
    
}